from typing import Any, Dict, List, Union

from github.GithubObject import NonCompletableGithubObject
from github.Repository import Repository

class RepositoryPreferences(NonCompletableGithubObject):
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def preferences(self) -> Dict[str, List[Dict[str, Union[bool, int]]]]: ...
    @property
    def repository(self) -> Repository: ...
