from typing import Any, Dict, List, Union

from github.GithubObject import NonCompletableGithubObject

class SelfHostedActionsRunner(NonCompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def id(self) -> int: ...
    @property
    def name(self) -> str: ...
    @property
    def os(self) -> str: ...
    @property
    def status(self) -> str: ...
    @property
    def busy(self) -> bool: ...
    def labels(self) -> List[Dict[str, Union[str, int]]]: ...
